<?php
require_once 'custom/PxRoutines/NewsFeedLogic/Collector/MobileSync/Helper/MobileSyncContact.php';
require_once 'custom/PxRoutines/NewsFeedLogic/Provider/MobileNotify.php';

class MobileReSync{

	CONST DEFAULT_FILTER_OWNER = 1;
	CONST DEFAULT_FILTER_FAVORITE = 0;
	CONST DEFAULT_FILTER_FOLLOWING = 0;
	CONST DEFAULT_FILTER_TEAM = 0;

	public function refreshContactsByRegistration($gcmRegistrationId){
		global $current_user;
		$gcmRegistrationIds = array($gcmRegistrationId);

		$this->cleanUpOldGCMMessages($gcmRegistrationIds);
		$this->refreshSubscribedContactsByUserAndRegistrations($current_user->id, $gcmRegistrationIds);
	}

	/**
	 * @param $userId Refresh Contacts for all Registrations
	 */
	public function refreshSubscribedContactsByUser($userId){
		$gcmRegistrations = $this->getRegistrationsByUser($userId);
		$gcmRegistrationIds = $this->getRegistrationIds($gcmRegistrations);

		//delete all messages if total refresh is obligatory
		$this->cleanUpOldMessages($userId);
		$this->cleanUpOldGCMMessages($gcmRegistrationIds);
		$this->refreshSubscribedContactsByUserAndRegistrations($userId, $gcmRegistrationIds);
		$this->mobileNotifyAboutChanges($gcmRegistrations);
	}

	public function cleanUpAllUserMessages($userId){
		$this->cleanUpOldMessages($userId);
		$this->cleanUpOldGCMMessagesByUser($userId);
	}

	protected function mobileNotifyAboutChanges($gcmregistrations){
		$mobileNotifier = new MobileNotify();
		$mobileNotifier->sendNotification($gcmregistrations);
	}

	protected function refreshSubscribedContactsByUserAndRegistrations($userId, $gcmRegistrationIds){
		$contactIds = $this->publishSubscribedContactsForUser($userId);
		foreach($contactIds as $contactId){
			$contact = BeanFactory::getBean('Contacts', $contactId);
			$mobileSyncContact = new MobileSyncContact($contact);

			foreach($gcmRegistrationIds as $gcmRegistrationId){
				$this->publishMessageToUser($mobileSyncContact->getPreparedMessage(), $userId, $gcmRegistrationId, $contactId);
			}
		}
	}

	protected function getRegistrationsByUser($userId){
		$query = new SugarQuery();
		$query->from(BeanFactory::getBean('PX_GCMRegistrations'));
		$query->select(array('id', 'registration_token'));
		$query->where()->equals('assigned_user_id', $userId);

		return $query->execute();
	}
	protected function getRegistrationIds($registrations){
		$registrationIds = array();
		foreach($registrations as $row){
			$registrationIds[] = $row['id'];
		}

		return $registrationIds;
	}

	protected function publishSubscribedContactsForUser($userId){
		$contactIds = array();
		$userSubscriptions = $this->getUserSubscriptions($userId);
		foreach($userSubscriptions as $subscription){
			$filters = $this->getSubscriptionFilters($subscription);
			$contactIds = array_merge($contactIds, $this->getFilteredContactsForUser($userId, $filters));
		}
		return $contactIds;
	}

	protected function getUserSubscriptions($userId){
		$query = $this->getUserSubscriptionsQuery($userId);
		$subscriptions = $query->execute('array');
		if(empty($subscriptions)){
			$subscriptions = $this->createSubscriptionIfNotExists($userId);
		}
		return $subscriptions;
	}

	protected function getUserSubscriptionsQuery($userId){
		$query = new SugarQuery();
		$query->select(array(
			'id',
			'filter_owner',
			'filter_favorite',
			'filter_following',
			'filter_team',
		));
		$query->from(BeanFactory::getBean('PX_GCMSubscriptions'));
		$query->where()->equals('assigned_user_id', $userId);

		return $query;
	}

	protected function createSubscriptionIfNotExists($userId){
		$px_gcmsubscription = BeanFactory::getBean('PX_GCMSubscriptions');
		$px_gcmsubscription->assigned_user_id = $userId;
		$px_gcmsubscription->filter_owner = self::DEFAULT_FILTER_OWNER;
		$px_gcmsubscription->filter_favorite = self::DEFAULT_FILTER_FAVORITE;
		$px_gcmsubscription->filter_following = self::DEFAULT_FILTER_FOLLOWING;
		$px_gcmsubscription->filter_team = self::DEFAULT_FILTER_TEAM;
		$px_gcmsubscription->reloadMessages = false;
		$px_gcmsubscription->save();

		return array(
			array(
				'id' => $px_gcmsubscription->id,
				'filter_owner' => $px_gcmsubscription->filter_owner,
				'filter_favorite' => $px_gcmsubscription->filter_favorite,
				'filter_following' => $px_gcmsubscription->filter_following,
				'filter_team' => $px_gcmsubscription->filter_team,
			)
		);
	}

	protected function getSubscriptionFilters($subscription){
		$filters = array();

		if(!empty($subscription['filter_owner'])){
			$filters[] = '$owner';
		}
		if(!empty($subscription['filter_favorite'])){
			$filters[] = '$favorite';
		}
		if(!empty($subscription['filter_following'])){
			$filters[] = '$following';
		}
		if(!empty($subscription['filter_team'])){
			$filters[] = '$teams';
		}

		return $filters;
	}

	protected function getFilteredContactsForUser($userId, $filters){
		$queries = $this->getContactsQueriesForFilters($userId, $filters);
		if(empty($queries)){
			return array();
		}
		$contactIds = array();
		foreach($queries as $query){
			$rows = $query->execute('array');
			foreach($rows as $row){
				$id = $row['id'];
				$contactIds[$id] = 1;
			}
		}
		return array_keys($contactIds);
	}

	protected function getContactsQueriesForFilters($userId, $filters){
		$queries = array();
		if(in_array('$owner', $filters)){
			$queries[] = $this->getAssignedContactsQuery($userId);
		}
		if(in_array('$following', $filters)){
			$queries[] = $this->getFollowedContactsQuery($userId);
		}
		if(in_array('$favorite', $filters)){
			$queries[] = $this->getFavoritesContactsQuery($userId);
		}
		if(in_array('$teams', $filters)){
			$queries[] = $this->getTeamContactsQuery($userId);
		}
		return $queries;
	}

	protected function getAssignedContactsQuery($userId){
		$query = new SugarQuery();
		$query->select(array(
			'contacts.id',
		));
		$query->from(BeanFactory::getBean('Contacts'));
		$query->where()->equals('assigned_user_id', $userId);
		return $query;
	}

	protected function getFollowedContactsQuery($userId){
		$query = new SugarQuery();
		$query->select(array(
			'contacts.id',
		));
		$query->from(BeanFactory::getBean('Contacts'));

        $joinTableSQL = "(SELECT * from subscriptions WHERE deleted=0 AND parent_type='Contacts' AND created_by = '$userId')";
        $plJta = 'px_following_link';
        $query->join[$plJta] = $query->joinTable($joinTableSQL, array('alias' => $plJta));
        $query->join[$plJta]->on()->equalsField($plJta . '.parent_id', 'contacts.id');

		return $query;
	}

	protected function getFavoritesContactsQuery($userId){
		$query = new SugarQuery();
		$query->select(array(
			'contacts.id',
		));
		$query->from(BeanFactory::getBean('Contacts'));

        $joinTableSQL = "(SELECT * from sugarfavorites WHERE deleted=0 AND module='Contacts' AND created_by = '$userId')";
        $plJta = 'px_favorite_link';
        $query->join[$plJta] = $query->joinTable($joinTableSQL, array('alias' => $plJta));
        $query->join[$plJta]->on()->equalsField($plJta . '.record_id', 'contacts.id');

		return $query;
	}

	protected function getTeamContactsQuery($userId){
		$query = new SugarQuery();
		$query->select(array(
			'contacts.id',
		));

		$query->from(BeanFactory::getBean('Contacts'), array('team_security' => true));
		return $query;
	}

	protected function publishMessageToUser($message, $userId, $gcmRegistrationId, $contact_id){
		$gcmMessage = BeanFactory::getBean('PX_GCMMessages');
		$gcmMessage->name = "initial message for sync";
		$gcmMessage->message_type = 'mobile_sync';
		$gcmMessage->assigned_user_id = $userId;
		$gcmMessage->gcmregistration_id = $gcmRegistrationId;
		$gcmMessage->parent_type = 'Contacts';
		$gcmMessage->parent_id = $contact_id;
		$gcmMessage->message = $message;
		$gcmMessage->save(false);
	}

	//GCM MESSAGES
	protected function cleanUpOldGCMMessages($registrationIds){
		$messageQuery = $this->getOldGCMMessagesQuery($registrationIds);
		$gcmMessage = BeanFactory::getBean('PX_GCMMessages');
		foreach($messageQuery->execute() as $message){
			$gcmMessage->mark_deleted($message['id']);
		}
	}

	protected function getOldGCMMessagesQuery($registrationIds){
		$query = new SugarQuery();
		$query->from(BeanFactory::getBean('PX_GCMMessages'));
		$query->select(array('id'));
		$query->where()->in('gcmregistration_id', $registrationIds);
		return $query;
	}
	protected function cleanUpOldGCMMessagesByUser($userId){
		$messageQuery = $this->getOldGCMMessagesByUserQuery($userId);
		$gcmMessage = BeanFactory::getBean('PX_GCMMessages');
		foreach($messageQuery->execute() as $message){
			$gcmMessage->mark_deleted($message['id']);
		}
	}

	protected function getOldGCMMessagesByUserQuery($userId){
		$query = new SugarQuery();
		$query->from(BeanFactory::getBean('PX_GCMMessages'));
		$query->select(array('id'));
		$query->where()->in('assigned_user_id', $userId);
		return $query;
	}

	public function cleanUpHistoryGCMMessages(){
		$date = $this->getDateToIndicateHistoryGCMMessages();

		$query = new SugarQuery();
		$query->from(BeanFactory::getBean('PX_GCMMessages'));
		$query->select(array('id'));
		$query->where()->lt('date_modified', $date);

		$gcmMessage = BeanFactory::getBean('PX_GCMMessages');
		foreach($query->execute() as $message){
			$gcmMessage->mark_deleted($message['id']);
		}
	}

	protected function getDateToIndicateHistoryGCMMessages(){
		$dateTime = new DateTime();
		$timeZone = new DateTimeZone('UTC');
		$dateTime->setTimezone($timeZone);
		$dateTime->sub(new DateInterval('P30D'));
		return $dateTime->format('Y-m-d H:i:s');
	}

	//USER_NEWS MESSAGES
	protected function cleanUpOldMessages($userId){
		$messageQuery = $this->getOldMessagesQuery($userId);
		$userMessage = BeanFactory::getBean('PX_UserNews');
		foreach($messageQuery->execute() as $message){
			$userMessage->mark_deleted($message['id']);
		}
	}

	protected function getOldMessagesQuery($userId){
		$query = new SugarQuery();
		$query->from(BeanFactory::getBean('PX_UserNews'));
		$query->select(array('id'));
		$query->where()->equals('assigned_user_id', $userId);
		return $query;
	}

}