<?php

class MobileSyncActivities{
	//static $repeatedActivitiesInCreate;

	//this logic is for hooks, because relations don't load correctly at that moment
	//removed activities from hook
	protected $unlinkedActivities = array();
	//added activities with repeated parent id
	protected $linkedRepeatedActivities = array();

	static public $newActivities = array();


	protected $activityList = array();


	static $activityDisplayStylesMap = array(
		'Calls' => array(
			'type' => 'Call',
			'symbol' => 'Cl',
			'color' => '#FF3333',
		),
		'Meetings' => array(
			'type' => 'Meeting',
			'symbol' => 'Me',
			'color' => '#2FD419',
		),
		'Tasks' => array(
			'type' => 'Task',
			'symbol' => 'Ta',
			'color' => '#1986D4',
		),
		'Emails' => array(
			'type' => 'Email',
			'symbol' => 'Em',
			'color' => '#BC30E3',
		)
	);

	public function __construct($contact){
		$this->contact = $contact;
	}

	public function getList(){

		if(!empty($this->activityList))
			return $this->activityList;

		$this->getRelatedCalls()
			->getRelatedMeetings()
			->getRelatedTasks()
			->getRelatedEmails();

		$this->searchForRepeatedActivities();


		return $this->formatActivities();
	}

	public function setUnlinkedRecords($unlinkedRecords){
		$this->unlinkedActivities = !empty($unlinkedRecords) ? $unlinkedRecords : array();
		return $this;
	}

	protected function getUnlinkedRecords(){
		return $this->unlinkedActivities;
	}

	public function setLinkedRepeatedActivities($repeatedLinkedRecords){
		$this->linkedRepeatedActivities = !empty($repeatedLinkedRecords) ? $repeatedLinkedRecords : array();
		return $this;
	}

	public function searchForRepeatedActivities(){

		if(!empty($this->linkedRepeatedActivities)){
			foreach($this->linkedRepeatedActivities as $moduleName => $item){
				foreach($item as $itemId => $itemData){
					$this->activityList[$moduleName][$itemId] = array(
						'name' => $itemData['name'],
						'activity_date' => $this->getTimestampFromString($itemData['activity_date']),
					);
				}
			}
		}
	}


	private function getTimestampFromString($value){
		$date = new DateTime($value);
		$timestamp = $date->getTimestamp();
		return $timestamp;
	}

	protected function getRelatedCalls(){
		$this->contact->load_relationship('calls');
		$relatedCalls = $this->formatRelatedActivities($this->contact->calls->getRelatedModuleName(), $this->contact->calls->get());
		$this->activityList = array_merge($this->activityList, $relatedCalls);
		return $this;
	}
	protected function getRelatedMeetings(){
		$this->contact->load_relationship('meetings');
		$relatedMeetings = $this->formatRelatedActivities($this->contact->meetings->getRelatedModuleName(), $this->contact->meetings->get());
		$this->activityList = array_merge($this->activityList, $relatedMeetings);
		return $this;
	}
	protected function getRelatedTasks(){
		$this->contact->load_relationship('tasks');
		$taskIds = $this->contact->tasks->get();

		foreach($this->getTasksFromContactParentLink() as $additionalTask){
			if(in_array($additionalTask['id'], $taskIds))
				continue;
			$taskIds[] = $additionalTask['id'];
		}


		$relatedTasks = $this->formatRelatedActivities($this->contact->tasks->getRelatedModuleName(), $taskIds);
		$this->activityList = array_merge($this->activityList, $relatedTasks);
		return $this;
	}

	protected function getTasksFromContactParentLink(){
		$query = new SugarQuery();
		$query
			->from(BeanFactory::getBean('Tasks'))
			->select(array('id'));
		$query->where()->equals('parent_type', 'Contacts');
		$query->where()->equals('parent_id', $this->contact->id);

		return $query->execute();
	}

	protected function formatRelatedActivities($moduleName, $activityIds){
	    if(!empty(self::$newActivities[$moduleName])){
	        foreach(self::$newActivities[$moduleName] as $id){
	            if(!in_array($id, $activityIds)){
                    $activityIds[] = $id;
                }
            }
        }
		$beans = $this->getActivityQuery($moduleName, $activityIds);
		$activities = array();

		$unlinkedActivities = $this->getUnlinkedRecords();
		foreach($beans as $bean){
			//Deleted links
			if(!empty($unlinkedActivities[$bean['id']]['module']) && $unlinkedActivities[$bean['id']]['module'] == $moduleName)
				continue;

			//Current Links
			if(!empty($bean['date_start'])) {
				$activities[$moduleName][$bean['id']] =
					array(
						'name' => $bean['name'],
						'activity_date' => $this->getTimestampFromString($bean['date_start']),
					);
			}
		}
		return $activities;
	}

	protected function getActivityQuery($moduleName, $activityIds){
		if(empty($activityIds))
			return array($moduleName => array());
		$query = new SugarQuery();
		$query
			->from(BeanFactory::getBean($moduleName))
			->select(array('id', 'name', 'date_start'));
		$query->where()->in('id', $activityIds);

		return $query->execute();
	}

	protected function getRelatedEmails(){
	    global $timedate;
		$emails = array();
		$unlinkedActivities = $this->getUnlinkedRecords();

		$this->contact->load_relationship('emails');
		$emailIds = $this->contact->emails->get();

		$this->contact->load_relationship('archived_emails');
        $archivedEmailIds = $this->contact->archived_emails->get();

        $emailIds = array_merge($emailIds, $archivedEmailIds);
        if(!empty(self::$newActivities['Emails']))
            $emailIds = array_merge($emailIds, self::$newActivities['Emails']);
        $emailIds = array_unique($emailIds);

        foreach($emailIds as $k => $id){
            if(!empty($unlinkedActivities[$id]) && $unlinkedActivities[$id]['module'] == 'Emails')
                unset($emailIds[$k]);
        }

        $emailsRawData = $this->getRelatedEmailsRawData($emailIds);

        $activities = array();
        foreach($emailsRawData as $email){
            if(!empty($email['date_sent'])){
                if(empty($email['date_sent']))
                    continue;

                $activities['Emails'][$email['id']] =
                    array(
                        'name' => $email['name'],
                        'activity_date' => $this->getTimestampFromString($email['date_sent']),
                    );

            }
        }

		if(!empty($activities))
			$this->activityList = array_merge($this->activityList, $activities);
		return $this;
	}

    protected function getRelatedEmailsRawData($emailsIds){
        if(empty($emailsIds))
            return array('Emails' => array());
        $query = new SugarQuery();
        $query
            ->from(BeanFactory::getBean('Emails'))
            ->select(array('id', 'name', 'date_sent'));
        $query->where()->in('id', $emailsIds);

        return $query->execute();
    }

	protected function formatActivities(){
		$formattedResult = array();
		foreach($this->activityList as $moduleName => $item){
			foreach($item as $itemData){

				$activityAdditionalData = !empty(self::$activityDisplayStylesMap[$moduleName]) ? self::$activityDisplayStylesMap[$moduleName] : array();
				if(empty($activityAdditionalData))
					continue;

				$formattedResult[] = array_merge($activityAdditionalData,
					array(
						'name' => $itemData['name'],
						'activity_date' => $itemData['activity_date'],
					)
				);
			}
		}

		$this->activityList = $formattedResult;
		return $this->activityList;
	}

}