<?php

require_once 'custom/PxRoutines/NewsFeedLogic/Collector/MobileSync/Helper/MobileSyncActivities.php';

class MobileSyncContact{

	protected $contact;
	protected $hookArguments;
	protected $accountName;
	protected $activities = null;

	protected $favoritedByUserIds;
	protected $followedByUserIds;
	protected $additionalFollowingUsers = array();
	protected $teamUserIds;

	protected $data = array();


	public function __construct($contact){
		$this->setContact($contact);
	}

	protected function setContact($contact){
		if(is_string($contact))
			$this->contact = BeanFactory::getBean('Contacts', $contact);
		else
			$this->contact = $contact;
	}

	public function getContact(){
		return $this->contact;
	}

	public function setHookArguments($arguments){
		$this->hookArguments = $arguments;
		return $this;
	}

	public function getPreparedMessage(){

		if($this->isDeleted())
			return $this->getDeletedContactMessage();
		else{

			return $this->getUpdatedContactMessage();
		}

	}

	protected function isDeleted(){
		return $this->contact->deleted;
	}

	protected function getUpdatedContactMessage(){
		if(empty($this->data)){
			$message =
				array(
					'id' => $this->contact->id,
					'name' => $this->contact->full_name,
					'account_name' => $this->getAccountName(),
					'title' => $this->contact->title,
					'phones'=> array(
						array(
							'type'=> 'home',
							'number'=> $this->contact->phone_home,
						),
						array(
							'type'=> 'mobile',
							'number'=> $this->contact->phone_mobile,
						),
						array(
							'type'=> 'work',
							'number'=> $this->contact->phone_work,
						),
						array(
							'type'=> 'other',
							'number'=> $this->contact->phone_other,
						),
						array(
							'type'=> 'fax',
							'number'=> $this->contact->phone_fax,
						),
					),
					'emails' => $this->getEmailAddresses(),
					'activities' => $this->getActivities(),
				);

			$this->data = json_encode($message);
		}
		return $this->data;
	}
	protected function getDeletedContactMessage(){
		if(empty($this->data)){
			$this->data =
				array(
					'id' => $this->contact->id,
				);
		}
		return $this->data;
	}

	protected function getAccountName(){
		if(!isset($this->accountName))
			$this->setAccountName();
		return $this->accountName;
	}

	protected function setAccountName(){
		if(
			!empty($this->hookArguments['dataChanges']['account_id']) || empty($this->hookArguments['isUpdate']) ||
			(empty($this->contact->account_name) && !empty($this->contact->account_id)))
			$this->accountName = BeanFactory::getBean('Accounts', $this->contact->account_id);
		$this->accountName = $this->contact->account_name;
	}

	protected function getEmailAddresses(){
		$emails = array();
		foreach($this->contact->email as $email){
			$emails[] = array(
				'type' => 'work',
				'email_address' => $email['email_address']
			);
		}
		return $emails;
	}

	public function setActivities($activities = null){
		if(empty($activities)){
			$this->activities = new MobileSyncActivities($this->contact);
		}else
			$this->activities = $activities;
	}

	protected function getActivities(){
		if(empty($this->activities))
			$this->setActivities();

		return $this->activities->getList();
	}

	public function getFavoritedByUserIds(){
		if(!isset($this->favoritedByUserIds))
			$this->setFavoritedByUserIds();
		return $this->favoritedByUserIds;
	}


	public function getFollowedByUserIds(){
		if(!isset($this->followedByUserIds))
			$this->setFollowedByUserIds();
		return $this->followedByUserIds;
	}

	public function getTeamUserIds(){
		if(!isset($this->teamUserIds))
			$this->setTeamUserIds();
		return $this->teamUserIds;
	}

	public function checkOwner($userId){
		return !empty($this->contact->assigned_user_id) && $this->contact->assigned_user_id == $userId;
	}

	public function checkFavorite($userId){
		return in_array($userId,$this->getFavoritedByUserIds());
	}

	public function checkFollowing($userId){
		return in_array($userId, $this->getFollowedByUserIds());
	}

	public function checkTeams($userId){
		return in_array($userId, $this->getTeamUserIds());
	}

	protected function setFavoritedByUserIds(){
		$this->contact->load_relationship('favorite_link');
		$this->favoritedByUserIds = $this->contact->favorite_link->get();
	}

	protected function setFollowedByUserIds(){
		$this->contact->load_relationship('following_link');
		$this->followedByUserIds = $this->contact->following_link->get();
		$this->followedByUserIds = array_merge($this->followedByUserIds, $this->additionalFollowingUsers);
	}

	public function setAdditionalFollowingUsers($userId){
		$this->additionalFollowingUsers[] = $userId;
	}

	protected function setTeamUserIds(){
		require_once('modules/Teams/TeamSet.php');
		$teamSetBean = new TeamSet();

		$teams = $teamSetBean->getTeams($this->contact->team_set_id);
		foreach($teams as $team){
			$team->load_relationship('users');
			foreach($team->users->get() as $userId){
				$this->teamUserIds[$userId] = $userId;
			}
		}
//		if(empty($teamsUsers))
//			$this->teamUserIds = array();
	}
}